package de.intevation.flys.client.server;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import de.intevation.artifacts.common.utils.ClientProtocolUtils;
import de.intevation.artifacts.common.utils.CreationFilter;

import de.intevation.artifacts.httpclient.exceptions.ConnectionException;
import de.intevation.artifacts.httpclient.http.HttpClient;
import de.intevation.artifacts.httpclient.http.HttpClientImpl;
import de.intevation.artifacts.httpclient.utils.ArtifactNamespaceContext;
import de.intevation.artifacts.httpclient.utils.XMLUtils;

import de.intevation.flys.client.shared.exceptions.ServerException;
import de.intevation.flys.client.shared.model.Artifact;

import de.intevation.flys.client.shared.model.Recommendation;

/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class ArtifactHelper {

    private static final Logger logger = Logger.getLogger(ArtifactHelper.class);


    /** The error message key that is thrown if an error occured while artifact
     * creation.*/
    public static final String ERROR_CREATE_ARTIFACT = "error_create_artifact";

    /**
     * Name of the factory to generate a GaugeDischargeCurveArtifact
     */
    private static final String GAUGE_DISCHARGE_CURVE_ARTIFACT = "gaugedischargecurve";


    private ArtifactHelper() {
    }


    /**
     * @param factory ArtifactFactory to use.
     */
    public static Artifact createArtifact(
        String         serverUrl,
        String         locale,
        String         factory,
        Recommendation recommendation)
    throws ServerException
    {
        logger.debug("ArtifactHelper.create");

        String         uuid;
        String         ids;
        CreationFilter filter;

        if (recommendation != null) {
            uuid   = recommendation.getMasterArtifact();
            ids    = recommendation.getIDs();
            filter = convertFilter(recommendation.getFilter());
        }
        else {
            uuid   = null;
            ids    = null;
            filter = null;
        }

        Document create = ClientProtocolUtils.newCreateDocument(
            factory, uuid, ids, filter);

        return sendCreate(serverUrl, locale, create);

    }

    /**
     * Creates a new GaugeDischargeCurverArtifact
     *
     * @param river the name of the river
     * @param reference the reference id of the gauge (official number)
     */
    public static Artifact createGaugeDischargeCurveArtifact(
            String serverUrl,
            String locale,
            String river,
            Long   reference)
    throws ServerException
    {
        Document create = ClientProtocolUtils.newCreateDocument(
                GAUGE_DISCHARGE_CURVE_ARTIFACT);

        XMLUtils.ElementCreator ec = new XMLUtils.ElementCreator(
            create,
            ArtifactNamespaceContext.NAMESPACE_URI,
            ArtifactNamespaceContext.NAMESPACE_PREFIX);

        Element root = create.getDocumentElement();

        Element eriver = ec.create("river");
        ec.addAttr(eriver, "name", river);

        Element egauge = ec.create("gauge");
        ec.addAttr(egauge, "reference", reference.toString());

        root.appendChild(eriver);
        root.appendChild(egauge);

        return sendCreate(serverUrl, locale, create);
    }

    /**
     * Sends a create document to the artifact server
     */
    private static Artifact sendCreate(
            String   serverUrl,
            String   locale,
            Document doc)
    throws ServerException
    {
        HttpClient client = new HttpClientImpl(serverUrl, locale);

        try {
            return (Artifact) client.create(doc, new FLYSArtifactCreator());
        }
        catch (ConnectionException ce) {
            logger.error(ce, ce);
        }

        throw new ServerException(ERROR_CREATE_ARTIFACT);
    }


    /**
     * Create CreationFilter from Recommendation.Filter.
     */
    public static CreationFilter convertFilter(Recommendation.Filter filter) {

        if (filter == null) {
            return null;
        }

        CreationFilter cf = new CreationFilter();

        Map<String, List<Recommendation.Facet>> outs = filter.getOuts();

        for (Map.Entry<String, List<Recommendation.Facet>> entry:
            outs.entrySet()) {
            List<Recommendation.Facet> rfs = entry.getValue();
            List<CreationFilter.Facet> cfs =
                new ArrayList<CreationFilter.Facet>(rfs.size());
            for (Recommendation.Facet rf: rfs) {
                cfs.add(new CreationFilter.Facet(rf.getName(), rf.getIndex()));
            }
            cf.add(entry.getKey(), cfs);
        }

        return cf;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
